/*
   $Id: scoring.h 649 2011-10-31 13:57:42Z manu $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/

#ifndef __SCORING_H__
#define __SCORING_H__

#define N_INFTY -50

#include "ptypes.h"
#include "masque.h"
#include "Bank.h"
#include "bitcount.h"


/*
   Commentaire a ajouter ici sur ScoreMatrix - MANU

*/

class ScoreMatrix: public Singleton<ScoreMatrix> {
protected:
    ScoreMatrix(): __mask(Mask::Instance())
    {
		Bank& bank= Bank::Instance();
		const float etotal  = (float)bank.GetNbWords();
        int i,size=0;
        for (i=ROOT_LEVEL;i<=__mask.GetLeafLevel();i++)
        {
            size += __mask.GetLength(i);
            m[i] = etotal*pow(0.25,size);
            s[i] = size;
        }
		BitCount::Instance();
    };
    
public:
	friend class Singleton<ScoreMatrix>;
	short operator [](short i) { return s[i];};

private:
	const Mask& __mask;
    double m[MAX_LEAF_LEVEL];
    short s[MAX_LEAF_LEVEL];
};

/*
   Commentaire a ajouter ici sur MatchConstants

*/
int __Overlap(const Mask& m, int i, int j, int k);
int __Overlap_bw(const Mask& m, int i, int j, int k);
class MatchConstants: public Singleton<MatchConstants> {
protected:
    MatchConstants ():  bank(Bank::Instance()),
						__mask(Mask::Instance()),
						score_matrix(ScoreMatrix::Instance()),
						Overlap_score(__InitOverlapScore(__Overlap)),
						Overlap_score_bw(__InitOverlapScore(__Overlap_bw)) {};
	~MatchConstants()
	{
		__FreeFromScoring ( Overlap_score );
		__FreeFromScoring ( Overlap_score_bw );
	};

public:
	friend class Singleton<MatchConstants>;
	
private:	
	Bank& bank;
	const Mask& __mask;

public:											// TODO - METTRE CA EN PRIVE ?
	ScoreMatrix& score_matrix;
	short*** Overlap_score;
	short*** Overlap_score_bw;

private:
	short*** __InitOverlapScore(int (overlap)(const Mask&, int, int, int k));
	void __FreeFromScoring(short***);
	//short*** __init_overlap_score_bw();
	//int __overlap    (const Mask& m, int i, int j, int k);
	//int __overlap_bw (const Mask& m, int i, int j, int k);
};




/*
   Commentaire a ajouter ici sur AlignConstants

*/
class AlignConstants : public Singleton<AlignConstants> {

    public:
		friend class Singleton<AlignConstants>;
		// GetScore, GetScoreSevere - NOTE strand's values are 0/1, NOT -1/1 !!!
	    int GetScore       (uint8_t strand01, unsigned char i,unsigned char j) const { return __score[strand01][i][j]; };
		int GetScoreSevere (uint8_t strand01, unsigned char i,unsigned char j) const { return __score_severe[strand01][i][j]; };
	    //int GetScore       (int8_t strand, unsigned char i,unsigned char j) const { return (strand==1?__score[0][i][j]:__score[1][i][j]); };
		//int GetScoreSevere (int8_t strand, unsigned char i,unsigned char j) const { return (strand==1?__score_severe[0][i][j]:__score_severe[1][i][j]); };

    private:
	    AlignConstants();  // See ptypes.h for the default parameters set
	    ~AlignConstants();

     	const Parameters& __parameters;
		//int X1,X2;
		//int alpha,beta;
		int match,mismatch,nmatch;
		
   		int ***__score;
		int ***__score_severe;	
	    int *** __InitScoringScheme(int, int, int );
	    int *** __InitScoringSchemeSevere(int, int, int );
	    int ** __InitScoringSchemeStrand(int, int, int, int );
	    int ** __InitScoringSchemeSevereStrand(int, int, int, int );
//	    int ** __InitScoringIndels(int , int );
	    
	    void __Free(void ***p);
		//void __FreeFromScoring(void ** p);
};


#endif
